
/* 
 * Get duration in hours.
 */

double GetDuration(TChain * chain) 
{
    // branches
    UInt_t M0, M1, M2;
    chain->SetBranchAddress("M0", &M0);
    chain->SetBranchAddress("M1", &M1);
    chain->SetBranchAddress("M2", &M2);
    vector<ULong64_t> T0(10), T1(10), T2(10);
    chain->SetBranchAddress("T0", T0.data());
    chain->SetBranchAddress("T1", T1.data());
    chain->SetBranchAddress("T2", T2.data());

    // find tmin and tmax
    int shifts = 0;
    double shift = 0;
    double Tp = 0, TFirst = 0, TLast = 0;    
    for (int i=0; i<chain->GetEntries(); i++) 
    {
        chain->GetEntry(i);

        double T00 = 0, T10 = 0, T20 = 0;
        if (M0 > 0) T00 = T0[0];  
        if (M1 > 0) T10 = T1[0]; 
        if (M2 > 0) T20 = T2[0];  

        double T = TMath::Max(T00, TMath::Max(T10, T20));

        if (Tp > T) {
            shift += Tp;
            shifts++;
        }
        if (i == 0) TFirst = T + shift;
        if (i == chain->GetEntries() - 1) TLast = T + shift;
        Tp = T;
    }

    double seconds = (TLast - TFirst) * 1E-8;
    double minutes = seconds / 60.;
    double hours = minutes / 60.;

    return hours;
}

/* 
 * Locate and fit 1.6-MeV gamma peak
 */

void FitGammaPeak(TH1D * h, double& counts, double& e0, double& sigma) 
{
}


/* 
 * Plot transmission as a function of magnetic-field setting.
 */

void TransmissionExp(bool veto = false)
{
    // directories
    TString expdir = "/home/oliskir/Desktop/f20/data/sorted/new/";
    TString simdir = "070617/";

    // gamma detection efficiency
    const double gammaEff = 0.54E-6;

    // beta spectrum integration limits
    const double emin = 100;
    const double emax = 5400;
    const int bins = 530;
    
    // number of magnetic field settings
    const int N = 10;
    vector<int> bmax = {15, 20, 25, 30, 35, 40, 44, 46, 48, 50};

    // runs at each setting
    vector<int> runs[N];
    runs[0] = {366};
    runs[1] = {332};
    runs[2] = {367};
    runs[3] = {333};
    runs[4] = {368};
    runs[5] = {334};
    runs[6] = {369};
    runs[7] = {360};
    runs[8] = {361, 362, 363, 364, 365, 370, 371, 372, 373, 374, 375, 376, 379, 380};
    runs[9] = {358};

    // background run (337)
    int rBackgr = 337;
    TChain * cBackgr = new TChain("data");    
    cBackgr->Add(expdir+Form("*0%i*", rBackgr));    
    TH1F * hBackgr0 = new TH1F("hBackgr0", "hBackgr0", bins, emin, emax);
    cBackgr->Draw("E0 >> hBackgr0", "Z0 != 10");
    double hoursBackgr = GetDuration(cBackgr);
    
    cout << endl;
    cout << " Background run:" << endl;
    cout << Form(" * Run:      %i", rBackgr) << endl;
    cout << Form(" * Entries:  %lli", cBackgr->GetEntries()) << endl;
    cout << Form(" * Duration: %.1f hours", hoursBackgr) << endl;
    cout << endl;

    // canvas
    TCanvas * c1 = new TCanvas("c1");

    // loop over magnetic field settings
    int i = 0;
    for (auto& b : bmax) 
    {
        cout << " Bmax: " << 0.01*b << " T" << endl;
    
        // add files
        TChain * cExp = new TChain("data");    
        for (auto& r : runs[i]) {
            TString fname = expdir + Form("*0%i*", r);
            cExp->Add(fname);    
        }
        
        // duration of experimental run
        double hours = GetDuration(cExp);
        cout << Form(" Duration: %.1f", hours) << endl; 

        // scale background spectrum
        TH1D * hBackgr = (TH1D*)hBackgr0->Clone("hBackgr");
        hBackgr->Scale(hours/hoursBackgr);

        // integrate background spectrum
        int binx1     = hBackgr->GetXaxis()->FindBin(emin);
        int binx2     = hBackgr->GetXaxis()->FindBin(emax);
        double backgr = hBackgr->Integral(binx1, binx2);

        // singles histograms
        TString nb = Form("hb%i", i);        
        TH1D * hb  = new TH1D(nb, nb, bins, emin, emax);
        TString ng = Form("hg%i", i);        
        TH1D * hg  = new TH1D(ng, ng, 1000, 0, 3000);

        // draw
        cExp->Draw("E0 >> " + nb,  "Z0 != 10");
        cExp->Draw("E2 >> " + ng,  "Z2 != 10");
        
        // integrate beta spectrum
        double betas = hb->Integral(binx1, binx2);
        betas -= backgr;
        
        // integrate 1.6-MeV peak in singles spectrum
        double gammas, centroid, sigma;
        FitGammaPeak(hg, gammas, centroid, sigma);

        // coincidence histogram
        TString nbg = Form("hbg%i", i);        
        TH1F * hbg  = new TH1F(nbg, nbg, 1000, 0, 3000);
        
        // draw
        double egmin = centroid - 3*sigma;
        double egmax = centroid + 3*sigma;
        cExp->Draw("E0 >> " + nbg, Form("Z0 != 10 && Z2 != 10 && E2 >= %f && E2 <= %f", egmin, egmax));

        // integrate coincidence beta spectrum
        double coincidences = hbg->Integral(binx1, binx2);

        // simulated beta spectrum
        TString fname = simdir + Form("b%i.root", b);
        TFile * fSim = new TFile(fname, "READ");
        TTree * tSim = (TTree*)fSim->Get("Detector");

        // histogram 
        TString nbSim = Form("hbSim%i", i);        
        TH1D * hbSim  = new TH1D(nbSim, nbSim, bins, emin, emax);

        // draw
        tSim->Draw("EDepSignal >> " + nbSim,  "");

        // scaling factor
        double decays = gammas / gammaEff;
        double norm   = ((TParameter<double>*)(tSim->GetUserInfo()->FindObject("NORMALIZATION")))->GetVal(); 
        hbSim->Scale(decays/norm);        
        
        // integrate
        double betasSim = hbSim->Integral(binx1, binx2);

        cout << Form(" Background:   %.0f", backgr) << endl; 
        cout << Form(" Betas:        %.0f", betas) << endl; 
        cout << Form(" Gammas:       %.0f", gammas) << endl; 
        cout << Form(" Coincidences: %.0f", coincidences) << endl; 
        cout << Form(" Simulated betas: %.0f", betasSim) << endl; 

        cin.get();
        i++;
    }
}
